// Program to find Dijkstra's shortest path using 
// priority_queue in STL 
//#include<bits/stdc++.h> 
#include <map>
#include <list>
#include <queue>
#include <iostream>
#include <SFML/Graphics.hpp>
using namespace std;
# define INF 0x3f3f3f3f 

bool debug = true;

// iPair ==>  Integer Pair 
typedef pair<int, int> iPair;

//This is to help print out the priority queue
/*template <class T, class S, class C>
S& Container(priority_queue<T, S, C>& q) {
	struct HackedQueue : private priority_queue<T, S, C> {
		static S& Container(priority_queue<T, S, C>& q) {
			return q.* & HackedQueue::c;
		}
	};
	return HackedQueue::Container(q);
}*/


// This class represents a directed graph using 
// adjacency list representation 
class Graph {
	int V;    // No. of vertices 

	// In a weighted graph, store vertex dest and weight 
	// pair for every edge from a particular vertex
	// The constructor will allocate memory for this and assign the memory location of the first one to adj
	list<iPair>* adj; //maybe list<iPair,predecessor>?

public:
	Graph(int V);  // Constructor 

	// function to add an edge to graph 
	void addEdge(int u, int v, int w);

	// prints shortest path from s 
	void shortestPath(int s);
};

// Constructor allocates memory for adjacency list 
Graph::Graph(int n) : V(n) {
	adj = new list<iPair>[V]; //allocate enough memory for V iPairs on the heap and assign the address of the first to adj
}

// Vertices u and v each get a new valence with weight w. u: "right back atchoo"
void Graph::addEdge(int u, int v, int w) {
	adj[u].push_back(make_pair(v, w));
	adj[v].push_back(make_pair(u, w));
}

// Prints shortest paths from src to all other vertices 
void Graph::shortestPath(int src) {
	// Create a priority queue to store vertices that 
	// are being preprocessed. This is weird syntax in C++. 
	// Refer below link for details of this syntax 
	// https://www.geeksforgeeks.org/implement-min-heap-using-stl/ 
	// The first parameter is required and the second too, since we need to specify the third is not the default "less"
	priority_queue< iPair, vector <iPair>, greater<iPair> > pq;
	//vector<int>& tasks = Container(pq);
	
	// Create a vector for distances and initialize all 
	// distances as infinite (INF) 
	vector<int> dist(V, INF); //{INF, INF, ... , INF}
	vector<int> pred(V, -1); // The value in the ith position is the index of the predecessor of the vertex with index i.
	// Insert source itself in priority queue and initialize 
	// its distance as 0. This primes the pump.
	
	pq.push(make_pair(0, src));
	dist[src] = 0;

	/* Looping till priority queue becomes empty (or all
	  distances are not finalized) */
	while (!pq.empty()) {
		// The first vertex in pair is the minimum distance 
		// vertex, extract it from priority queue. 
		// vertex label is stored in second of pair (it 
		// has to be done this way to keep the vertices 
		// sorted distance (distance must be first item 
		// in pair) 
		
		if (debug) {
			auto copyq = pq;
			cout << "priority queue is ";
			while (copyq.size()) {
				iPair temp = copyq.top();
				cout << '(' << temp.first << ", " << temp.second << ')';
				if(copyq.size()>0) cout << ", ";
				copyq.pop();
			}
			cout << endl;
			cout << "Distances: {";
			for (int i : dist) cout << i << " ";
			cout << '}' << endl;
		}

		int u = pq.top().second; //get vertex label so we can search and relax paths from there
		pq.pop(); //eliminate that vertex from the priority queue

		// iterator 'i' is used to add all adjacent vertices of a vertex 
		list<iPair>::iterator i; // This is a breadth first search
		for (i = adj[u].begin(); i != adj[u].end(); ++i) {
			// Get vertex label and weight of current adjacent 
			// of u. 
			int v = (*i).first; // arrow operator?
			int weight = (*i).second;

			//  If there is shorter path to v through u then relax the dist
			if (dist[v] > dist[u] + weight) { //dist[v] contains the currently discovered shortest distance from src to v
				// Updating predecessor and distance of v 
				pred[v] = u;
				if (debug) {
					cout << "Relaxing " << '(' << v << ", " << dist[v] << ") to " << "(" << v << ", " << dist[u] + weight << ")\n";
					cout << "Predecessors: ";
					for (int k : pred) cout << k << " ";
					cout << endl;
				}
				dist[v] = dist[u] + weight;
				pq.push(make_pair(dist[v], v)); //push onto priority queue
			}
		}
	}

	// Print shortest distances stored in dist[] 
	printf("Vertex   Distance from Source\n");
	for (int i = 0; i < V; ++i)
		cout << i << " <---> " << dist[i] << '\n';
		//printf("%d \t\t %d\n", i, dist[i]);
}

// Driver program to test methods of graph class 
int main() {
	// create the graph given in above fugure 
	int V = 9;
	Graph g(V);

	//  making above shown graph 
	g.addEdge(0, 1, 4);
	g.addEdge(0, 7, 8);
	g.addEdge(1, 2, 8);
	g.addEdge(1, 7, 11);
	g.addEdge(2, 3, 7);
	g.addEdge(2, 8, 2);
	g.addEdge(2, 5, 4);
	g.addEdge(3, 4, 9);
	g.addEdge(3, 5, 14);
	g.addEdge(4, 5, 10);
	g.addEdge(5, 6, 2);
	g.addEdge(6, 7, 1);
	g.addEdge(6, 8, 6);
	g.addEdge(7, 8, 7);

	g.shortestPath(0);

	return 0;
}